from enum import Enum


class ShapeType(Enum):
    TYPE_UNSPECIFIED = 0
    TEXT_BOX = 1
    RECTANGLE = 2
    ROUND_RECTANGLE = 3
    ELLIPSE = 4
    ARC = 5
    BENT_ARROW = 6
    BENT_UP_ARROW = 7
    BEVEL = 8
    BLOCK_ARC = 9
    BRACE_PAIR = 10
    BRACKET_PAIR = 11
    CAN = 12
    CHEVRON = 13
    CHORD = 14
    CLOUD = 15
    CORNER = 16
    CUBE = 17
    CURVED_DOWN_ARROW = 18
    CURVED_LEFT_ARROW = 19
    CURVED_RIGHT_ARROW = 20
    CURVED_UP_ARROW = 21
    DECAGON = 22
    DIAGONAL_STRIPE = 23
    DIAMOND = 24
    DODECAGON = 25
    DONUT = 26
    DOUBLE_WAVE = 27
    DOWN_ARROW = 28
    DOWN_ARROW_CALLOUT = 29
    FOLDED_CORNER = 30
    FRAME = 31
    HALF_FRAME = 32
    HEART = 33
    HEPTAGON = 34
    HEXAGON = 35
    HOME_PLATE = 36
    HORIZONTAL_SCROLL = 37
    IRREGULAR_SEAL_1 = 38
    IRREGULAR_SEAL_2 = 39
    LEFT_ARROW = 40
    LEFT_ARROW_CALLOUT = 41
    LEFT_BRACE = 42
    LEFT_BRACKET = 43
    LEFT_RIGHT_ARROW = 44
    LEFT_RIGHT_ARROW_CALLOUT = 45
    LEFT_RIGHT_UP_ARROW = 46
    LEFT_UP_ARROW = 47
    LIGHTNING_BOLT = 48
    MATH_DIVIDE = 49
    MATH_EQUAL = 50
    MATH_MINUS = 51
    MATH_MULTIPLY = 52
    MATH_NOT_EQUAL = 53
    MATH_PLUS = 54
    MOON = 55
    NO_SMOKING = 56
    NOTCHED_RIGHT_ARROW = 57
    OCTAGON = 58
    PARALLELOGRAM = 59
    PENTAGON = 60
    PIE = 61
    PLAQUE = 62
    PLUS = 63
    QUAD_ARROW = 64
    QUAD_ARROW_CALLOUT = 65
    RIBBON = 66
    RIBBON_2 = 67
    RIGHT_ARROW = 68
    RIGHT_ARROW_CALLOUT = 69
    RIGHT_BRACE = 70
    RIGHT_BRACKET = 71
    ROUND_1_RECTANGLE = 72
    ROUND_2_DIAGONAL_RECTANGLE = 73
    ROUND_2_SAME_RECTANGLE = 74
    RIGHT_TRIANGLE = 75
    SMILEY_FACE = 76
    SNIP_1_RECTANGLE = 77
    SNIP_2_DIAGONAL_RECTANGLE = 78
    SNIP_2_SAME_RECTANGLE = 79
    SNIP_ROUND_RECTANGLE = 80
    STAR_10 = 81
    STAR_12 = 82
    STAR_16 = 83
    STAR_24 = 84
    STAR_32 = 85
    STAR_4 = 86
    STAR_5 = 87
    STAR_6 = 88
    STAR_7 = 89
    STAR_8 = 90
    STRIPED_RIGHT_ARROW = 91
    SUN = 92
    TRAPEZOID = 93
    TRIANGLE = 94
    UP_ARROW = 95
    UP_ARROW_CALLOUT = 96
    UP_DOWN_ARROW = 97
    UTURN_ARROW = 98
    VERTICAL_SCROLL = 99
    WAVE = 100
    WEDGE_ELLIPSE_CALLOUT = 101
    WEDGE_RECTANGLE_CALLOUT = 102
    WEDGE_ROUND_RECTANGLE_CALLOUT = 103
    FLOW_CHART_ALTERNATE_PROCESS = 104
    FLOW_CHART_COLLATE = 105
    FLOW_CHART_CONNECTOR = 106
    FLOW_CHART_DECISION = 107
    FLOW_CHART_DELAY = 108
    FLOW_CHART_DISPLAY = 109
    FLOW_CHART_DOCUMENT = 110
    FLOW_CHART_EXTRACT = 111
    FLOW_CHART_INPUT_OUTPUT = 112
    FLOW_CHART_INTERNAL_STORAGE = 113
    FLOW_CHART_MAGNETIC_DISK = 114
    FLOW_CHART_MAGNETIC_DRUM = 115
    FLOW_CHART_MAGNETIC_TAPE = 116
    FLOW_CHART_MANUAL_INPUT = 117
    FLOW_CHART_MANUAL_OPERATION = 118
    FLOW_CHART_MERGE = 119
    FLOW_CHART_MULTIDOCUMENT = 120
    FLOW_CHART_OFFLINE_STORAGE = 121
    FLOW_CHART_OFFPAGE_CONNECTOR = 122
    FLOW_CHART_ONLINE_STORAGE = 123
    FLOW_CHART_OR = 124
    FLOW_CHART_PREDEFINED_PROCESS = 125
    FLOW_CHART_PREPARATION = 126
    FLOW_CHART_PROCESS = 127
    FLOW_CHART_PUNCHED_CARD = 128
    FLOW_CHART_PUNCHED_TAPE = 129
    FLOW_CHART_SORT = 130
    FLOW_CHART_SUMMING_JUNCTION = 131
    FLOW_CHART_TERMINATOR = 132
    ARROW_EAST = 133
    ARROW_NORTH_EAST = 134
    ARROW_NORTH = 135
    SPEECH = 136
    STARBURST = 137
    TEARDROP = 138
    ELLIPSE_RIBBON = 139
    ELLIPSE_RIBBON_2 = 140
    CLOUD_CALLOUT = 141
