""" "elementGroup": {
    object (Group)
  },
  "shape": {
    object (Shape)
  },
  "image": {
    object (Image)
  },
  "video": {
    object (Video)
  },
  "line": {
    object (Line)
  },
  "table": {
    object (Table)
  },
  "wordArt": {
    object (WordArt)
  },
  "sheetsChart": {
    object (SheetsChart)
  }"""
